<?php
namespace Xoogu\CurrencyConversion;
class Admin extends CurrencyConversion{
    
    public $mainPluginFile;
	protected $_CountryFromIP_Admin;
    
    /**
     *Initialise the plugin
     *@param wpdb $wpdb Database connection
     *@param string $sBasePath Base path to this plugin, used for including add-on files etc.
     *@param \Xoogu\CountryFromIP\ICountryFromIP $countryFromIP (optional) CountryFromIP instance for geo lookup
     *@param \Xoogu\CountryFromIP\Admin $countryFromIP_Admin (optional) CountryFromIP admin instance for viewing and updating geo options
     */
    public function __construct(\wpdb $wpdb, $sBasePath, \Xoogu\CountryFromIP\ICountryFromIP $countryFromIP=null, \Xoogu\CountryFromIP\Admin $countryFromIP_Admin=null)
	{
        parent::__construct($wpdb, $sBasePath, $countryFromIP);
		$this->_CountryFromIP_Admin = $countryFromIP_Admin;
        //register the activation and deactivation methods
        $mainPluginFile=$this->mainPluginFile=$this->_sBasePath.'/xoogu-currency-conversion.php';
        register_activation_hook( $mainPluginFile, array($this, 'activate'));
        register_deactivation_hook( $mainPluginFile, array($this, 'deactivate'));
        //register_uninstall_hook( __FILE__, array('\Xoogu\CurrencyConversion\Admin', 'uninstall'));
		
		if (empty($this->options['networkOnly'])) {
			//multisite new blog added
			add_action( 'wpmu_new_blog', 'new_site_added', 10, 6);
			//add admin menu
			add_action('admin_menu', array($this, 'add_admin_menu'));
		}
        add_action('network_admin_menu', array($this, 'add_admin_menu'));
		
        //action hook for the cron
        add_action('xoogu_currency_conversion_update', array($this, 'updateRates'));
        
        //on post save
        add_action( 'save_post', array($this, 'onPostSave'), 9999 );
        
        load_plugin_textdomain('xoogu-currency-conversion', false, dirname(plugin_basename(__FILE__)).'/lang/');
		
		
		//$this->updateRates();
        
    }
	
    
    /**
     *Adds the plugin settings page to the Settings menu
     */
    public function add_admin_menu(){
		//Add a new menu item under Settings
        $pageTitle=sprintf(__('%s Currency Conversion','xoogu-currency-conversion'), 'Xoogu');
		$pluginFile = plugin_basename($this->mainPluginFile);
		if (is_plugin_active_for_network($pluginFile)) {
			$hook_suffix = add_submenu_page( 'settings.php', $pageTitle, $pageTitle, 'manage_options', 'xoogu_currency_conversion', array($this, 'display_page') );
			//add settings page link in plugins list
			add_filter('network_admin_plugin_action_links_'.$pluginFile, array($this, 'plugin_action_links_network'));
		}
		if (empty($this->options['networkOnly']) && !is_network_admin()) {
			$hook_suffix = add_options_page( $pageTitle, $pageTitle, 'manage_options', 'xoogu_currency_conversion', array($this, 'display_page') );
			//Using registered $page handle to hook stylesheet loading
			//add settings page link in plugins list
			add_filter('plugin_action_links_'.$pluginFile, array($this, 'plugin_action_links'));
		}
		//$hook_suffix won't be set when we're in network admin area and the plugin isn't network activated
		if (!empty($hook_suffix)) {
			add_action( 'load-' . $hook_suffix, array($this, 'admin_load') );
		}
    }
    
    
    /**
     *Load scripts, css etc. on the settings page
     */
    public function admin_load(){
        wp_enqueue_style('xoogu_currency_conversion_admin', plugins_url('admin.css', $this->mainPluginFile), null, self::$sVersion);
        wp_enqueue_script('xoogu_currency_conversion_admin', plugins_url('admin.js', $this->mainPluginFile), array('jquery'), self::$sVersion);
        wp_localize_script( 'xoogu_currency_conversion_admin', 'xcc_L10n', array(
			'show_opts'  => __('Show advanced options', 'xoogu-currency-conversion' ),
			'hide_opts' => __('Hide advanced options', 'xoogu-currency-conversion' )
		) );
    }
    
    
    /**
     *Validate and save the POSTed settings when the settings form is submitted
     */
    public function validate_settings($input){
        
        //check nonce & referrer
        if (wp_verify_nonce($_POST['_xoogu_currencyConversion_nonce'], 'save_settings') !== 1) {
            wp_die(__('A valid nonce was not found.','xoogu-currency-conversion') );
        }
        if(!current_user_can('manage_options')){
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="http://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','xoogu-currency-conversion') );
        }
        $errors = array();
        
        //from
        if(!empty($input['from'])){
            if($this->getCurrencyByCode($input['from'])){
                $this->options['from']=(string)$input['from'];
            }else{
                $errors['from'] = sprintf(__('%s is not a valid currency code', 'xoogu-currency-conversion'), $input['from']);
            }
        }else{
            unset($this->options['from']);
        }
        
        //dec_places
        if($input['dec_places'] !== ''){
            if(!$this->isValidInteger($input['dec_places']) || (int)$input['dec_places']<0){
                $errors['dec_places'] = 'dec_places must be an integer of zero or greater (or blank)';
            }else{
                $this->options['dec_places']=(int)$input['dec_places'];
            }
        }else{
            unset($this->options['dec_places']);
        }
        
        //static
        if(!empty($input['static'])){
            $this->options['static']=1;
        }else{
            unset($this->options['static']);
        }
        
        //updateFreq
        $freqs = wp_get_schedules();
        if (empty($freqs[$input['updateFreq']])) {
            $errors['updateFreq'] = sprintf(__('%s is not a valid update frequency', 'xoogu-currency-conversion'), $input['updateFreq']);
        } elseif ($this->options['updateFreq'] != (string)$input['updateFreq']) {
            $this->options['updateFreq']=(string)$input['updateFreq'];
			$rescheduleCron = true;
        }
        
        //format
        $this->options['format']=(string)$input['format'];
        
        //addOns
        if (!empty($input['addOns'])) {
            //activate any addons set to on
            foreach ($input['addOns'] as $addOn => $val) {
                if (empty($this->options['addOns'][$addOn])) {
					$addOnPath = $this->_sBasePath.'/add-ons/'.$addOn;
					if(file_exists($addOnPath)){
						$this->options['addOns'][$addOn] = true;
						include_once($addOnPath);
						do_action('activate_' . plugin_basename($addOnPath), is_plugin_active_for_network(plugin_basename($this->mainPluginFile)));
					}
				}
            }
        }
        if (!empty($this->options['addOns'])) {
            //deactivate any previously active addons now set to off
            foreach ($this->options['addOns'] as $addOn => $val) {
                if (!$input['addOns'][$addOn]) {
                    unset($this->options['addOns'][$addOn]);
                }
            }
        }
        
        //should be impossible to have any errors apart from dec_places not being an int
        if (!empty($errors)) {
            return $errors;
        }
		
		
		//networkOnly
		//check network activated and we're being called from network settings page by network admin user
		if (is_plugin_active_for_network(plugin_basename($this->mainPluginFile)) && is_network_admin() && is_super_admin()){
			global $wpdb;
			if (!empty($input['networkOnly'])) {
				//If networkOnly wasn't previously set but is now, we should remove the separate options from each of the blogs
				if (empty($this->options['networkOnly'])) {
					//remove options from network sites
					$current = $wpdb->blogid;
					// Loop through the blogs
					$blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
					foreach ($blogids as $blog_id) {
						switch_to_blog($blog_id);
						//we deactivate and delete the db options for the site, but don't do a site uninstall as we want to preserve any static rates
						$this->_deactivate_site();
						delete_option('xoogu_currency_conversion');
					}
					//switch back to the original blog
					switch_to_blog($current);
					//update option
					$this->options['networkOnly']=true;
					//since we will have removed the cron for the main blog above (when running _deactivate_site()), we'll need to add it again
					$rescheduleCron=true;
				}
				update_site_option('xoogu_currency_conversion', $this->options);
				if (!empty($rescheduleCron)) {
					$this->rescheduleCron();
				}
			} else {
				//If network only was previously set but is now unticked, we need to remove the sitewide option
				if (!empty($this->options['networkOnly'])) {
					//unset the networkOnly setting so it won't be set when we update the options of the indivdual blogs
					unset($this->options['networkOnly']);
					//remove the site option
					delete_site_option('xoogu_currency_conversion');
					//since cron will have previously been for the main blog only, we'll now need to add it to all blogs
					$rescheduleCron=true;
				}
				//add options to network sites
				$current = $wpdb->blogid;
				// Loop through the blogs
				$blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
				foreach ($blogids as $blog_id) {
					switch_to_blog($blog_id);
					update_option('xoogu_currency_conversion', $this->options);
					if (!empty($rescheduleCron)) {
						$this->rescheduleCron();
					}
				}
				//switch back to the original blog
				switch_to_blog($current);
			}
		} else {
			update_option('xoogu_currency_conversion', $this->options);
			if (!empty($rescheduleCron)) {
				$this->rescheduleCron();
			}
		}
    }
	
	
	/**
	 *Removes the current update hook from the cron then adds a new one (effectively just modifying the cron)
	 *Used when changing the interval period between when the update rates hook should run
	 */
	public function rescheduleCron() {
		//clear the current cron
		wp_clear_scheduled_hook( 'xoogu_currency_conversion_update' );
		$freqs = wp_get_schedules();
        //schedule the cron
        wp_schedule_event(time()+$freqs[$this->options['updateFreq']]['interval'],$this->options['updateFreq'],'xoogu_currency_conversion_update');
	}
    
    
    /**
     *Display the settings page
     *See also the duplicate in Xoogu_CurrencyConversion_Admin_MCE::display_mce_form
     */
    public function display_page(){
        if(!current_user_can('manage_options')){
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="http://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','xoogu-currency-conversion') );
        }
        //if the form was submitted, process the POST
		if(isset($_POST['submit'])){
			$errors = $this->validate_settings($_POST['options']);
			if(empty($errors)){
				?><div class="updated"><p><?php _e('Settings saved successfully.','xoogu-currency-conversion')?></p></div><?php
			}
			//set the form values to those that were submitted
			$formValues = $_POST['options'];
        }
        else{
            //otherwise set the form values to the saved options
            $formValues = $this->options;
        }
        include $this->_sBasePath.'/tpl/admin.php';
    }
    
    /**
     *Filter callback attached to plugin_action_links_(plugin filename) hook
     *Adds a settings link to the plugin in the plugins list
     *@param array $links The links to print for this plugin entry
     *@return array The links to print for this plugin entry
     */
    public function plugin_action_links($links){
        array_unshift($links, '<a href="'. get_admin_url(null, 'options-general.php?page=xoogu_currency_conversion') .'">'.__('Settings','xoogu-currency-conversion').'</a>');
        return $links;
    }
	
    
    /**
     *Filter callback attached to plugin_action_links_(plugin filename) hook
     *Adds a settings link to the plugin in the plugins list
     *@param array $links The links to print for this plugin entry
     *@return array The links to print for this plugin entry
     */
    public function plugin_action_links_network($links){
        array_unshift($links, '<a href="'. network_admin_url('settings.php?page=xoogu_currency_conversion') .'">'.__('Settings','xoogu-currency-conversion').'</a>');
        return $links;
    }
    
    
    /**
     *Gets a list of the installed addOns and their metadata
     *@param string $dir The directory to look in for the addOns
     *@return array Associative array of the addOns with their metadata
     */
    public function getAddOnsData($dir){
        $addOns = array();
        if (!is_dir($dir)) {
			return $addOns;
		}
        $handle = opendir($dir);
        while( false !== ($entry = readdir($handle)) ){
            if($entry == '.' || $entry == '..'){
                continue;
            }
            //if the addOn is located in its own directory, the addOn file should have the exact same name as the directory
            if(is_dir($dir.'/'.$entry)){
                $addOnFile = $entry.'/'.$entry.'.php';
            }elseif(is_file($dir.'/'.$entry)){
                $addOnFile = $entry;
            }
            if( $data = get_plugin_data( $dir.'/'.$addOnFile ) ){
                $addOns[$addOnFile]=$data;
            }
        }
        closedir($handle);
        
        return $addOns;
    }
	
	
	/**
	 *Removes the 'geoFunc' setting that was used in versions 2016-02-17 and lower. This setting is now saved separately for the CountryFromIP class.
     *@param bool $network Whether the plugin is being network activated
	 */
	protected function _updateTo20160218($network)
	{
		//check the version and update if necessary
		$version = get_site_option('xoogu_currency_conversion_version');
		if (!$version || !($version < '2016-02-18')) {
			return;
		}
		
		global $wpdb;
		
		//Do we already have an existing geofunc set?
		if ( ($geoFunc = !empty($this->options['geoFunc']) ? $this->options['geoFunc'] : null) ) {
			//If using the ajax add-on and geoip-detect plugin, then change the geo method to the 'geoip-detect-geolite-city' method (skips loading the geoip-detect plugin itself, and so works with SHORTINIT)
			if ($geoFunc == 'geoip-detect2'
				&& !empty($this->options['addOns']['ajax/ajax.php'])
				&& file_exists(ABSPATH.'wp-content/uploads/GeoLite2-City.mmdb')
				&& file_exists(ABSPATH.'wp-content/plugins/geoip-detect/vendor/maxmind-db/reader/src/MaxMind/Db/Reader.php')
				) {
				$geoFunc = 'geoip-detect-geolite-city';
			}
			//If being activated from the network page, then save the geo method as a site option
			if ($network) {
				$this->_CountryFromIP_Admin->saveSettings($geoFunc);
			}
			
			//remove geo method from this instance
			unset($this->options['geoFunc']);
			//update db to remove geo method from settings
			if (!is_multisite()) {
				update_option('xoogu_currency_conversion', $this->options);
				$this->_CountryFromIP_Admin->saveSettings($geoFunc);
			} elseif (!empty($this->options['networkOnly'])) {
				update_site_option('xoogu_currency_conversion', $this->options);
			}
		//If not and we're network activating, try and auto-detect and save the geofunc
		} elseif ($network) {
			$this->_CountryFromIP_Admin->detectAndSetGeoIP();
		}
		//Are we on multisite and not networkOnly? If so, then update the individual blogs to remove the geoFunc setting. 
		if (is_multisite() && empty($this->options['networkOnly'])) {
			$current = $wpdb->blogid;
			// Loop through the blogs
			$blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
			foreach ($blogids as $blog_id) {
				switch_to_blog($blog_id);
				$aOpts = get_option('xoogu_currency_conversion');
				//if no geo func was set for this blog, then nowt to do
				if (empty($aOpts['geoFunc'])) {
					continue;
				}
				//otherwise remove the geo func from the settings
				$geoFunc = $aOpts['geoFunc'];
				unset($aOpts['geoFunc']);
				update_option('xoogu_currency_conversion', $aOpts);
				//if we're not network activating the plugin we need to save the geo method for the new CountryFromIP class on a per blog basis
				if (!$network) {
					//If using the ajax add-on and geoip-detect plugin, then change the geo method to the 'geoip-detect-geolite-city' method (skips loading the geoip-detect plugin itself, and so works with SHORTINIT)
					if ($geoFunc == 'geoip-detect2'
						&& !empty($this->options['addOns']['ajax/ajax.php'])
						&& file_exists(ABSPATH.'wp-content/uploads/GeoLite2-City.mmdb')
						&& file_exists(ABSPATH.'wp-content/plugins/geoip-detect/vendor/maxmind-db/reader/src/MaxMind/Db/Reader.php')
						) {
						$geoFunc = 'geoip-detect-geolite-city';
					}
					$this->_CountryFromIP_Admin->saveSettings($geoFunc);
				}
			}
			//switch back to the original blog
			switch_to_blog($current);
		}
	}
	
	
	/**
	 *Versions prior to 1.2 used a different database schema, so we need to drop the existing tables if updating
	 */
	protected function _updateTo1dot2()
	{
		global $wpdb;
        $wpdb->show_errors();
        $tblCur=$this->_tblCur;
        $tblCou=$this->_tblCou;
		
		//check the version and update the database table if necessary
		$version = get_site_option('xoogu_currency_conversion_version');
		if($version && $version < '1.2'){
			//Data and table structure has been modified from previous version
			//Easiest thing to do is drop existing tables then make them anew
			
			//dbDelta doesn't support dropping tables (weird?) so use a standard query
			if( !$wpdb->query("DROP TABLE IF EXISTS `$tblCur`, `$tblCou`") ){
                //couldn't drop tables so die
                wp_die(new WP_Error('update', sprintf(__('Dropping of table %s or %s failed.','xoogu-currency-conversion'), $tblCur, $tblCou), $sql));
            }
		}
	}
    
    
    /**
     *Create the database table when the plugin is activated
     *@param bool $network Whether the plugin is being network activated
     */
    public function activate($network){
        //check the current has permission to activate the plugin
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        
        //check the nonce is valid (will automatically die if it is not)
        $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
        //$plugin = activate-plugin_xoogu-currency-conversion/xoogu-currency-conversion.php
        check_admin_referer("activate-plugin_{$plugin}");
		
		
		//Updates
		//Update for versions prior to 1.2
        $this->_updateTo1dot2();
		//Update for versions prior to 2016-02-18
		$this->_updateTo20160218($network);
        
        //update the wordpress table with the version (useful for checking when upgrading)
		//we can use update as this will add the entry if doesn't already exist
		update_site_option('xoogu_currency_conversion_version', self::$sVersion);
		
        
		//create the db tables and insert data
		global $wpdb;
        $wpdb->show_errors();
        $tblCur=$this->_tblCur;
        $tblCou=$this->_tblCou;
		
		
        //Currency details
		$this->_createCurrencyTable($tblCur);
		
		//Country to currency map table
        $this->_createCountryTable($tblCou);
        
        //multisite network activation
        if(is_multisite() && $network){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to network activate the plugin', 'xoogu-currency-conversion' ) );
            }
			//If plugin was previously network activated in network only mode, then deactivated, and now we're activating it again
			if( ($oldOpts = get_site_option('xoogu_currency_conversion_old')) && !empty($oldOpts) ) {
				$this->options = $oldOpts;
				delete_site_option('xoogu_currency_conversion_old');
				add_site_option('xoogu_currency_conversion', $this->options);
				//clean up cron and options of any blogs that may have had the plugin activated individually
				$current = $wpdb->blogid;
				// Loop through the blogs
				$blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
				foreach ($blogids as $blog_id) {
					switch_to_blog($blog_id);
					self::_deactivate_site();
					delete_option('xoogu_currency_conversion');
				}
				//switch back to the original blog
				switch_to_blog($current);
				//enable the cron again for the main blog so we have one cron running
				$this->rescheduleCron();
			} else {
				$current = $wpdb->blogid;
				// Loop through the blogs
				$blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
				foreach ($blogids as $blog_id) {
					switch_to_blog($blog_id);
					$this->_activate_site();
				}
				//switch back to the original blog
				switch_to_blog($current);
			}
        }
        //standard
        else{
            $this->_activate_site();
        }
        
        //create the network-wide option to store when the cron was last run
		//we set this to 0 regardless of whether it already exists or not, so that activating the plugin will force a rate update
        update_site_option('xoogu_currency_conversion_ratesLastUpdated', 0);
        //update the rates now (we don't use cron for the first update as it won't run straight away if using a real cron to trigger wp_cron)
        $this->updateRates();
    }
	
	
	/**
	 *Creates the wp_xoogu_currency_details db table and fills it with the currency info from the included currency_details.csv file
	 *@param string $tblCur The name of the table to use
	 */
	protected function _createCurrencyTable($tblCur)
	{
		global $wpdb;
		if($wpdb->get_var("SHOW TABLES LIKE '$tblCur'") == $tblCur) {
			return;
		}
		//create the table
		$sql = 'CREATE TABLE '.$tblCur.' (
				code CHAR(3) NOT NULL,
				symbol VARCHAR(6) NOT NULL,
				name VARCHAR(25) NOT NULL,
				symbol_entity_num VARCHAR(30) NOT NULL,
				symbol_entity_hex VARCHAR(30) NOT NULL,
				full_name VARCHAR(50) NOT NULL,
				dec_places TINYINT(1) NOT NULL,
				symbol_pref BOOLEAN NOT NULL,
				format_pref CHAR(2) NOT NULL,
				rate FLOAT UNSIGNED NOT NULL,
				PRIMARY KEY  (code),
				KEY (symbol)
		)';
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
		//unfortunately there seems no way to tell if dbDelta created the table successfully or not from its return value, so we'll have to manually check if the table was created
		if($wpdb->get_var("SHOW TABLES LIKE '$tblCur'") != $tblCur){
			//couldn't create table so die
			wp_die(new WP_Error('tblCurCreation', sprintf(__('Creation of table %s failed.','xoogu-currency-conversion'), $tblCur), array('title' => sprintf(__('Creation of table %s failed.','xoogu-currency-conversion'), $tblCur), 'sql' => $sql)));
		}
		//Fill the table
		$sql= "INSERT INTO $tblCur (code, symbol, name, symbol_entity_num, symbol_entity_hex, full_name, dec_places, symbol_pref, format_pref) VALUES ";
		if (($handle = fopen(plugin_dir_path(__FILE__).'/../../../currency_details.csv', 'r')) !== FALSE) {
			$data = fgets($handle);
			$sql.= "($data)";
			while (($data = fgets($handle)) !== FALSE) {
				$sql.= ", ($data)";
			}
			fclose($handle);
		}else{
			//couldn't open data file so die
			wp_die(new WP_Error('tblCurCreation', sprintf(__('Couldn\'t open data file %s.','xoogu-currency-conversion'), 'currency_details.csv') ));
		}
		if(!$wpdb->query($sql)){
			//couldn't fill table so die
			wp_die(new WP_Error('tblCurCreation', sprintf(__('Couldn\'t insert data into %s.','xoogu-currency-conversion'),$tblCur ),$sql ));
		}
	}
	
	
	/**
	 *Creates the wp_xoogu_currency_country db table and fills it with the country info from the included country_currency_map.csv file
	 *@param string $tblCou The name of the table to use
	 */
	protected function _createCountryTable($tblCou)
	{
		global $wpdb;
		if($wpdb->get_var("SHOW TABLES LIKE '$tblCou'") == $tblCou) {
			return;
		}
		//create the table
		//symbol_pos - 0 => before amount, 1 => after amount
		//dec_point and thousands_sep must be varchar and not char as mysql trims char fields on retrieval (so space will end up as an empty string)
		$sql = 'CREATE TABLE IF NOT EXISTS '.$tblCou.' (
				country CHAR(2) NOT NULL,
				currency CHAR(3) NOT NULL,
				dec_point VARCHAR(1) NOT NULL,
				thousands_sep VARCHAR(1) NOT NULL,
				symbol_pos BOOLEAN NOT NULL,
				PRIMARY KEY  (country,currency)
		)';
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
		//If the table wasn't created then die
		if($wpdb->get_var("SHOW TABLES LIKE '$tblCou'") != $tblCou) {
			//couldn't create table so die
			wp_die(new WP_Error('tblCouCreation', sprintf(__('Creation of table %s failed.','xoogu-currency-conversion'), $tblCou), $sql));
		}
		//Fill the table
		$sql= "INSERT INTO $tblCou VALUES ";
		if (($handle = fopen(plugin_dir_path(__FILE__).'/../../../country_currency_map.csv', 'r')) !== FALSE) {
			$data = fgets($handle);
			$sql.= "($data)";
			while (($data = fgets($handle)) !== FALSE) {
				$sql.= ", ($data)";
			}
			fclose($handle);
		}else{
			//couldn't open data file so die
			wp_die(new WP_Error('tblCouCreation', sprintf(__('Couldn\'t open data file %s.','xoogu-currency-conversion'), 'country_currency_map.csv') ));
		}
		
		if(!$wpdb->query($sql)){
			//couldn't fill table so die
			wp_die(new WP_Error('tblCouCreation', sprintf(__('Couldn\'t insert data into %s.','xoogu-currency-conversion'),$tblCou ),$sql ));
		}
	}
    
    
    /**
     *Creates the options entries in the db for the current site and schedules the cron for the current site
     *@param bool $networkOnly Whether the plugin is being activated in the multisite network only mode. Defaults to false.
     */
    protected function _activate_site($networkOnly=false){
        //default format: original text (USD 50)
        if(empty($this->options)){
            $this->options = array('format' => '%o (%a %c)', 'updateFreq' => 'daily');
            $this->_CountryFromIP_Admin->detectAndSetGeoIP();
        }
        if ($networkOnly) {
			add_site_option('xoogu_currency_conversion', $this->options);
		} else {
			add_option('xoogu_currency_conversion', $this->options);
		}
        //update the rates regularly
		//use reschedule cron as it doesn't hurt to try and remove any existing cron jobs first even if they don't exit
		//(they can exist if plugin was singly activated and is now being network activated)
        $this->rescheduleCron();
    }
    
    
    /**
     *Callback attached to the wpmu_new_blog hook
     *When a new blog is added to a multisite network, we need to check whether the plugin should be activated for this new blog
     *We don't need the params other than $blog_id, though for param details see http://codex.wordpress.org/Function_Reference/wpmu_create_blog
     */
    function new_site_added($blog_id, $user_id, $domain, $path, $site_id, $meta ) {
        global $wpdb;
        //only activate the plugin for the new site if it has been network activated
        if (is_plugin_active_for_network(plugin_basename($this->mainPluginFile))) {
            $current = $wpdb->blogid;
            switch_to_blog($blog_id);
            //run the plugin activation for the new site
            $this->_activate_site();
            switch_to_blog($current);
        }
    }
    
    
    /**
     *Remove cron and all database records when plugin is removed
     */
    public static function uninstall(){
        
        //check permissions
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        //check nonce
        check_admin_referer('bulk-plugins');        
        
        global $wpdb;
        //mysql table names (copied from __construct of main class, can't use non static properties in uninstall function)
        $tblCur = $wpdb->base_prefix.'xoogu_currency_details';
        $tblCou = $wpdb->base_prefix.'xoogu_currency_country';
        
        //multisite network uninstall (can only ever be a network uninstall in multisite)
        if(is_multisite()){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to delete the plugin', 'xoogu-currency-conversion' ) );
            }
			
			delete_site_option('xoogu_currency_conversion_old');
			
            $current = $wpdb->blogid;
            // Loop through the blogs
            $blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                self::_uninstall_site();
            }
            //switch back to the original blog
            switch_to_blog($current);
        }
        //standard
        else{
            self::_uninstall_site();
        }
        
        //remove the db tables
        $wpdb->query("DROP TABLE $tblCur");
        $wpdb->query("DROP TABLE $tblCou");
        
        //remove the ratesLastUpdated option used by the cron
        delete_site_option('xoogu_currency_conversion_ratesLastUpdated');
        //remove the plugin version from the db
        delete_site_option('xoogu_currency_conversion_version');
    }
    
    
    /**
     *Removes the per site database options used by the plugin
     */
    private static function _uninstall_site(){
        global $wpdb;
        
        //remove the update job from the cron
        self::_deactivate_site();
        
        //remove the options saved in the db
        delete_option('xoogu_currency_conversion');
        
        //remove any static rates saved as post_meta. We don't use a loop with delete_post_meta as suggested by the codex ( http://codex.wordpress.org/Function_Reference/delete_post_meta#Other_Examples ) as that would result in a lot of separate SQL queries and be slooow.
        if($wpdb->query('DELETE FROM '.$wpdb->postmeta." WHERE meta_key = '_xoogu_currency_conversion_rates'") === false){
            return new WP_Error('post_meta', __('Failed deleting post_meta', 'xoogu-currency-conversion'), $wpdb->last_error);
        }
    }
    
    
    /**
     *Remove cron when plugin is deactivated
     *@param bool $network If the plugin is being network deactivated
     */
    public function deactivate($network){
        //check the current has permission to activate the plugin
        if(!current_user_can('activate_plugins')){
            wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','xoogu-currency-conversion' ) );
        }
        
        //check the nonce is valid (will automatically die if it is not)
        $plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
        //$plugin = activate-plugin_xoogu-currency-conversion/xoogu-currency-conversion.php
        check_admin_referer( "deactivate-plugin_{$plugin}" );
        
        
        global $wpdb;
        
        //multisite
        if(is_multisite() && $network){
            if(!is_super_admin()){
                wp_die( __( 'You must be a super admin to network deactivate the plugin.', 'xoogu-currency-conversion' ) );
            }
			
			//If we're storing the plugin options sitewide, then we'll need to remove them when the plugin is network deactivated
			//Otherwise they'll persist in the db and when an individual blog enables the plugin it will find the site options
			//and use those rather than getting its own options
			if (!empty($this->options['networkOnly'])) {
				delete_site_option('xoogu_currency_conversion');
				add_site_option('xoogu_currency_conversion_old', $this->options);
			}
			
            $current = $wpdb->blogid;
            // Loop through the blogs
            $blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
			//If we weren't using network only mode we need to deactivate each blog
			if (empty($this->options['networkOnly'])) {
				foreach ($blogids as $blog_id) {
					switch_to_blog($blog_id);
					//only deactivate the plugin for the site if the site doesn't have it activated individually
					//we can't use is_plugin_active() as the plugin is still currently network activated. So we just use the relevant
					//part of the is_plugin_active function to check if the plugin is active for the current site
					if (!in_array( plugin_basename($this->mainPluginFile), (array) get_option( 'active_plugins', array() ) )) {
						self::_deactivate_site();
					}
				}
			} else {
				//If we were using network only mode we don't need to deactivate the indivdual blogs
				//But we do need to check for any blogs that may have had the plugin activated indivdually, before it was network activated
				//If the plugin is meant to be active on an individual blog, then we'll need to reactivate the plugin for that blog
				unset($this->options['networkOnly']);
				foreach ($blogids as $blog_id) {
					switch_to_blog($blog_id);
					//we can't use is_plugin_active() as the plugin is still currently network activated. So we just use the relevant
					//part of the is_plugin_active function to check if the plugin is active for the current site
					if (in_array( plugin_basename($this->mainPluginFile), (array) get_option( 'active_plugins', array() ) )) {
						$this->_activate_site();
					}
				}
			}
            //switch back to the original blog
            switch_to_blog($current);
        }
        //standard
        else{
            self::_deactivate_site();
        }
    }
    
    
    /**
     *Removes the per site cron
     */
    private static function _deactivate_site(){
        //remove the update job from the cron
        wp_clear_scheduled_hook('xoogu_currency_conversion_update');
    }
    
    
    /**
     *Gets the latest currency conversion rates from Yahoo and updates the database
     */
    public function updateRates(){
        //check the rates need updating
        //get the time the rates were last updated
        $lastRun = get_site_option('xoogu_currency_conversion_ratesLastUpdated');
        //get the registered cron frequency details
        $freqs = wp_get_schedules();
        //if the rates don't need updating yet (can happen if running the plugin on 2 or more sites in a multisite installation), then we don't need to do anything
        if(time() < $lastRun+$freqs[$this->options['updateFreq']]['interval']){
			return;
        }
        
        global $wpdb;
        $tblCur=$this->_tblCur;
		
		if ( ($get=wp_remote_get('http://finance.yahoo.com/webservice/v1/symbols/allcurrencies/quote?format=json')) && ($rateStr = wp_remote_retrieve_body($get)) ) {
			if($info = json_decode($rateStr)){
				
				//initialise an array to hold the values to be inserted
				$sqlParts=array();
				//get the currency codes from the database
				//we flip the array so we can check the code exists easily rather than having to use in_array()
				$codes = array_flip($this->getCurrencyList());
				//loop through the results from Yahoo
				foreach($info->list->resources as $currency){
					//parse the currency code (Yahoo data gives it like GBP=X, so we just need the first 3 chars to get the code)
					$code = substr($currency->resource->fields->symbol, 0, 3);
					//check the code is in the db
					if(isset($codes[$code])){
						//add the rate to the sql query
						$sqlParts[] = "('$code', ".(1/(float)$currency->resource->fields->price).") ";
					}
				}
				$sql = "INSERT INTO $tblCur (code, rate) VALUES ".implode(', ', $sqlParts).'ON DUPLICATE KEY UPDATE rate = VALUES(rate)';
				//update the database
				if($wpdb->query($sql) === false){
					return new WP_Error( 'updateRates', __('Failed updating the database table', 'xoogu-currency-conversion'), $wpdb->last_error );
				}
				//update the database to indicate the time the rates have been updated
				update_site_option('xoogu_currency_conversion_ratesLastUpdated', time());
			}
			else{
				return new WP_Error('updateRates', __('Failed decoding exchange rate json data from Yahoo', 'xoogu-currency-conversion'), $rateStr);
			}
		}
        else{
            return new WP_Error('updateRates', __('Failed fetching exchange rates from Yahoo', 'xoogu-currency-conversion'));
        }
    }
    
    
    /**
     *Saves the current rates as post meta for the current post
     *@param int $post_id The ID of the post being saved
     */
    public function saveStaticRates($post_id){
        global $wpdb;
        $tblCur=$this->_tblCur;
        //fetch the codes and rates from the db
        if(!($results = $wpdb->get_results("SELECT code, rate FROM $tblCur"))){
            return;
        }
        //format as an associative array with the currency code as the key
        $rates = array();
        foreach($results as $row){
            $rates[$row->code]=$row->rate;
        }
        //save the post meta to the db
        add_post_meta($post_id, '_xoogu_currency_conversion_rates', $rates, true);
    }
    
    
    /**
     *Called when post is saved. Checks to see if we have any shortcodes with the static attribute set to true
     *If we do, then will save the current rates as post meta
     *@param int $post_id The ID of the post being saved
     */
    public function onPostSave($post_id){
        
        //don't bother doing anything if just an autosave
        if (defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE){
            return;
        }
        
        global $post_data;
        //the content being saved may be stored in $_POST, $_GET, or $post_data.
        //Probably $_GET is only used for quick edits, where you can't edit the content anyway, so no need to check it
        //WP adds slashes to POST data, so now we need to remove them GRR!!!
        $post_content = wp_unslash(!empty($_POST['post_content']) ? $_POST['post_content'] : $post_data['post_content']);
        
        //if there's no content, then no point in looking for the shortcode
        if(empty($post_content)){
            return;
        }
        
        //if the rates have already been saved in the post meta, then there's nothing to do
        if(get_post_meta($post_id, '_xoogu_currency_conversion_rates', true)){
            return;
        }
        
        //if static is set in the global options, then save the rates if the content includes the shortcode
        if(!empty($this->options['static'])){
            if(has_shortcode($post_content, 'currencyConversion')){
                return $this->saveStaticRates($post_id);
            }
            return;
        }
        
        //check if we have any shortcodes with the static attribute
        //has_shortcode won't let you check if an attribute is set, so have to use a custom regex:
        $pattern = '/(?<!\\[)\\[currencyConversion[^\\]]* static=[\'"]?([^\'"]+)[\'"]?(?:\\]| .*\\])/U';
        $exists = preg_match_all($pattern, $post_content, $matches);
        //check our matches
        foreach($matches[1] as $val){
            //if static was set to true (case insenstive) or 1, then save the rates to the post meta
            if(!empty($val) && strcasecmp($val, 'true') === 0 || $val == '1'){
                return $this->saveStaticRates($post_id);
            }
        }
    }
    
}
