<?php
namespace Xoogu\CurrencyConversion;
class Admin_MCE{
	
	protected $_pearant;
	
    /**
     *Initialise the plugin
     *@param Xoogu_CurrencyConversion_Admin $pearant The parent object this object will 'inherit' from
     */
    public function __construct(\Xoogu\CurrencyConversion\Admin $pearant){
		$this->_pearant = $pearant;
		$this->_sBasePath = $pearant->getBasePath();
		//tiny mce (call directly rather than hooking into admin_init as we are being loaded from the current_screen hook, which is called after the admin_init hook)
		$this->mce_init();
		//quicktags and shared
		add_action('admin_enqueue_scripts', array($this, 'loadAdminjscss'));
		add_action('wp_ajax_xccsh_quicktags_dialog', array($this, 'quicktags_dialog'));
    }
    
	
	/**
	 *Loads the scripts and CSS needed to add the shortcode button and settings dialog for the tinyMCE and quicktags editors
	 */
	public function loadAdminjscss(){
		//quicktags
		//wordpress does include some jquery ui dialog css, but it is rather minimal and results in a transparaent div
		//wp_enqueue_style('wp-jquery-ui-dialog');
		wp_enqueue_style('jquery-ui', '//code.jquery.com/ui/1.10.3/themes/smoothness/jquery-ui.css');
		wp_enqueue_script('xoogu_currency_conversion_quicktags', plugins_url( 'quicktags.js' , $this->_sBasePath.'/a' ), array('jquery-ui-dialog', 'jquery','quicktags'), CurrencyConversion::$sVersion, true );
		wp_enqueue_style( 'xoogu_currency_conversion_mce', plugins_url( 'mce_quicktags_shared.css' , $this->_sBasePath.'/a' ), null, CurrencyConversion::$sVersion );
		
		//shared
		wp_enqueue_script('xoogu_currency_conversion_mce_quicktags_shared', plugins_url( 'mce_quicktags_shared.js' , $this->_sBasePath.'/a' ), array('jquery'), CurrencyConversion::$sVersion, true );
		wp_localize_script( 'xoogu_currency_conversion_mce_quicktags_shared', 'xcc_L10n', array(
			'dec_places_error'  => __("dec_places must be an integer of zero or greater (or blank)", 'xoogu-currency-conversion' ),
			'content_error' => __("Content must include an amount to be converted", 'xoogu-currency-conversion' ),
			'show_description' => __('Show description', 'xoogu-currency-conversion'),
			'hide_description' => __('Hide description', 'xoogu-currency-conversion'),
			'help_for_setting' => __('Help for this setting', 'xoogu-currency-conversion'),
		    'button_title' => __('Show this amount to visitors in their local currency', 'xoogu-currency-conversion'),
			'mce_plugin_longname' => sprintf(__('%s Currency Conversion shortcode', 'xoogu-currency-conversion'), 'Xoogu'),
			'dialog_title'  => sprintf(__('%s Currency Conversion shortcode settings', 'xoogu-currency-conversion'), 'Xoogu'),
			'cancel' => __('Cancel', 'xoogu-currency-conversion'),
			'insert' => __('Insert shortcode', 'xoogu-currency-conversion')
		) );
	}
    
	
    /**
     *Adds the filters needed to add the button to the TinyMCE Editor
     */
    public function mce_init() {
        add_filter( 'mce_external_plugins', array($this, 'mce_add_buttons') );
        add_filter( 'mce_buttons', array($this,'mce_register_buttons') );
    }
    
	
    /**
     *Adds the TinyMCE plugin javascript
     *@param Array $plugin_array Array of TinyMCE plugins to be loaded
     *@return Array Array of TinyMCE plugins to be loaded
     */
    public function mce_add_buttons( $plugin_array ) {
        $plugin_array['xoogu_currency_conversion'] = plugins_url( 'mce-button.js' , $this->_sBasePath.'/a' );
        return $plugin_array;
    }
    
	
    /**
     *Registers the button to display in the TinyMCE Editor
     *@param Array $plugin_array Array of TinyMCE buttons to be available
     *@return Array Array of TinyMCE buttons to be available
     */
    function mce_register_buttons( $buttons ) {
        array_push( $buttons, 'xoogu_currency_conversion' );
        return $buttons;
    }
    
	
    /**
     *Display the settings page
     *See also the partial duplicate in \Xoogu\CurrencyConversion\Admin::display_page
     */
    public function display_mce_form(){
		$pearant=$this->_pearant;
		include $this->_sBasePath.'/tpl/editor-shortcode-form.php';
    }
	
	
	/**
	 *Outputs the settings form the quicktags dialog AJAX call
	 */
	public function quicktags_dialog(){
		header('Content-type: text/html');
		$this->display_mce_form();
		exit();
	}
}
